@@ -1,11 +1,11 @@
+import logging
+
 from requests import Request, Session, hooks
 from requests.adapters import HTTPAdapter
-
+from urllib.parse import urlencode
 from twilio.http import HttpClient
-from twilio.http.response import Response
 from twilio.http.request import Request as TwilioRequest
-import logging
-from twilio.compat import urlencode
+from twilio.http.response import Response
 
 _logger = logging.getLogger('twilio.http_client')
 
@@ -14,7 +14,9 @@ class TwilioHttpClient(HttpClient):
     """
     General purpose HTTP Client for interacting with the Twilio API
     """
-    def __init__(self, pool_connections=True, request_hooks=None, timeout=None, logger=_logger, proxy=None, max_retries=None):
+
+    def __init__(self, pool_connections=True, request_hooks=None, timeout=None, logger=_logger, proxy=None,
+                 max_retries=None):
         """
         Constructor for the TwilioHttpClient
 
@@ -37,7 +39,7 @@ class TwilioHttpClient(HttpClient):
         if timeout is not None and timeout <= 0:
             raise ValueError(timeout)
         self.timeout = timeout
-        self.proxy = proxy
+        self.proxy = proxy if proxy else {}
 
     def request(self, method, url, params=None, data=None, headers=None, auth=None, timeout=None,
                 allow_redirects=False):
@@ -70,32 +72,46 @@ class TwilioHttpClient(HttpClient):
             'hooks': self.request_hooks
         }
 
-        if params:
-            self.logger.info('{method} Request: {url}?{query}'.format(query=urlencode(params), **kwargs))
-            self.logger.info('PARAMS: {params}'.format(**kwargs))
-        else:
-            self.logger.info('{method} Request: {url}'.format(**kwargs))
-        if data:
-            self.logger.info('PAYLOAD: {data}'.format(**kwargs))
+        self._log_request(kwargs)
 
         self.last_response = None
         session = self.session or Session()
-        if self.proxy:
-            session.proxies = self.proxy
         request = Request(**kwargs)
         self.last_request = TwilioRequest(**kwargs)
 
         prepped_request = session.prepare_request(request)
-        response = session.send(
-            prepped_request,
-            allow_redirects=allow_redirects,
-            timeout=timeout if timeout is not None else self.timeout,
-        )
 
-        self.logger.info('{method} Response: {status} {text}'.format(
-            method=method, status=response.status_code, text=response.text)
-        )
+        settings = session.merge_environment_settings(prepped_request.url, self.proxy, None, None, None)
 
-        self.last_response = Response(int(response.status_code), response.text)
+        settings['allow_redirects'] = allow_redirects
+        settings['timeout'] = timeout if timeout is not None else self.timeout
+
+        response = session.send(prepped_request, **settings)
+
+        self._log_response(response)
+
+        self.last_response = Response(int(response.status_code), response.text, response.headers)
 
         return self.last_response
+
+    def _log_request(self, kwargs):
+        self.logger.info('-- BEGIN Twilio API Request --')
+
+        if kwargs['params']:
+            self.logger.info('{} Request: {}?{}'.format(kwargs['method'], kwargs['url'], urlencode(kwargs['params'])))
+            self.logger.info('Query Params: {}'.format(kwargs['params']))
+        else:
+            self.logger.info('{} Request: {}'.format(kwargs['method'], kwargs['url']))
+
+        if kwargs['headers']:
+            self.logger.info('Headers:')
+            for key, value in kwargs['headers'].items():
+                # Do not log authorization headers
+                if 'authorization' not in key.lower():
+                    self.logger.info('{} : {}'.format(key, value))
+
+        self.logger.info('-- END Twilio API Request --')
+
+    def _log_response(self, response):
+        self.logger.info('Response Status Code: {}'.format(response.status_code))
+        self.logger.info('Response Headers: {}'.format(response.headers))
